!----------------------------- egs_rmgbh.f -----------------------------
!-----------------------------------------------------------------------
!  PROGRAMMERS: Y. Kirihara                                            *
!               Department of Accelerator Science,                     *
!               The Graduate University for Advanced Studies,          *
!               1-1, Oho, Tsukuba, Ibaraki, 305-0801                   *
!               Japan                                                  *
!                                                                      *
!               E-mail: kyoichi@post.kek.jp                            *
!-----------------------------------------------------------------------
      subroutine rmgbh(E0,k,ratio_mgbh)


      implicit none

      include 'include/egs5_h.f' ! COMMONs required by EGSD5 code
      include 'include/egs5_media.f'
      include 'include/egs5_brempr.f'
      include 'include/egs5_uphiot.f'
      include 'include/egs5_useful.f'

      real*8 AN,FSC,R0,RLC,Z_AB,Z_f,A_d,a,l_delta,delta,f_c,L_rad_d,
     *       L_rad,xi_bh,E,phi1_bh,phi2_bh,f1_bh,f2_bh,f3_bh,bhsigma
      real*8 k,E0,y,E_LPM,s_d,h,xi,s,phi,psi,G,sq2s1,ls1,
     *       mgsigma,ratio_mgbh
      real*8 h_bar,l_c,l_m_e,l_m,l_e,l_n,y_diel,max_y,k_p,gmm,omega,
     * GM,s_GM,phi_g,psi_g,dielsigma
c ------------------------------------------------------------------
c Branching ratio as a function of an element.
c ------------------------------------------------------------------
      ! for Branching ratio
      integer i
      real*8 rnnow                                       ! Arguments
      real*8 sum_pz,sum_z,r_pz(MXEL),
     *       ele_pdf(MXEL),ele_cdf(MXEL)
      real*8 Z,l_WA,RHO

      sum_pz = 0.d0
      sum_z  = 0.d0


      do i=1,nne(medium)
        sum_pz = sum_pz + pz(medium,i)
      end do

      do i=1,nne(medium)
        r_pz(i) = pz(medium,i)/sum_pz
        sum_z = sum_z + r_pz(i)*zelem(medium,i)*(zelem(medium,i)+1.)
      end do

      do i=1,nne(medium)
        ele_pdf(i) = r_pz(i)*zelem(medium,i)*(zelem(medium,i)+1.)/sum_z
      end do

c     -----------------------
c     Calculate cdf from pdf
c     -----------------------

      ele_cdf(1) = ele_pdf(1)
      do i=2,nne(medium)
        ele_cdf(i) = ele_cdf(i-1) + ele_pdf(i)
      end do
      call randomset(rnnow)
      do i=1,nne(medium)
        if (rnnow.le.ele_cdf(i)) then
          go to 100
        end if
      end do
100   continue

      Z    = zelem(medium,i)
      l_WA = wa(medium,i)
      RHO  = rhom(medium)

c -------------------- Start of EGS-BH cross section ---------------
c According to the EGS5 CODE SYSTEM p.37 - 54
c AN is explicitly written because PMCONS and uphiot cause conflict 
c because both contain PI
c FSC and RO are explicitly written here because dercon and useful 
c cause conflict because both contain RM
c ------------------------------------------------------------------
      AN = 6.02214199D+23          ! Avogadro's number
      FSC = 7.297360155253394E-003 ! fine structure constant
      R0 = 2.817943337446684E-013  ! Electron radius

      ! Eq. (2.53) from the EGS5 CODE SYSTEM
      a = FSC*Z
      f_c = a**2.*((1.+a**2.)**(-1.) + 0.20206 - 0.0369*a**2.
     * + 0.0083*a**4. - 0.002*a**6.)

      ! Eq. (2.57) from the EGS5 CODE SYSTEM
      if (Z.eq.1.) then
        L_rad = 5.310
        L_rad_d = 6.144
      else if (Z.eq.2.) then
        L_rad = 4.790
        L_rad_d = 5.621
      else if (Z.eq.3.) then
        L_rad = 4.740
        L_rad_d = 5.805
      else if (Z.eq.4.) then
        L_rad = 4.710
        L_rad_d = 5.924
      else
        L_rad = log(184.15*Z**(-1./3.))
        L_rad_d = log(1194.*Z**(-2./3.))
      end if

      ! Eq. (2.57) from the EGS5 CODE SYSTEM
      xi_bh = L_rad_d /(L_rad - f_c)

      ! No. 80 in Tab. 2.1 from the EGS5 CODE SYSTEM
      Z_AB = Z*(Z+xi_bh)*L_rad
      ! No. 40 in Tab. 2.1 from the EGS5 CODE SYSTEM
      Z_f = Z*(Z+xi_bh)*f_c

      ! when E0 > 50, A' = 1.
      A_d = 1.0

      E = E0 - k
      l_delta = k*RM/(2.*E0*E)
      delta = 136.*Z**(-1./3.)*2.*l_delta

c ----- first factor -----
      f1_bh = a_d * Z*(Z+xi_bh) /(4.*(z_ab-z_f))

c ----- phi1 and phi2 -----
      if(delta.le.1.) then
        phi1_bh = 20.867 - 3.242*delta + 0.625*delta**2.
        phi2_bh = 20.029 - 1.930*delta + 0.086*delta**2.
      else
        phi1_bh = 21.12 - 4.184*log(delta+0.952)
        phi2_bh = 21.12 - 4.184*log(delta+0.952)
      end if

c ----- second and third factor -----
      if (E0.ge.50.) then
        f2_bh = (1. + (E/E0)**2.) * (phi1_bh-4./3.*log(Z) - 4.*f_c)
        f3_bh = 2./3.*(E/E0) * (phi2_bh - 4./3.*log(Z) - 4.*f_c)
      else
        f2_bh = (1. + (E/E0)**2.) * (phi1_bh-4./3.*log(Z))
        f3_bh = 2./3.*(E/E0) * (phi2_bh - 4./3.*log(Z))
      end if

c ----- Original PEGS cross section for bremsstrahlung -----
      bhsigma = f1_bh * ( f2_bh - f3_bh )

c -------------------- End of BH cross section ------------------------

c -------------------- Start of LPM plus dielectric cross section -----
      RLC = rlcm(medium)
      E_LPM = 7.7e+6*RLC          ! E_LPM [MeV]
      ls1 = (Z**(1./3.)/191.)**2.
      y  = k/E0                   ! k/E0 photon / electron
      sq2s1 = sqrt(2.)*ls1

c ----- s'(x) -----
      s_d = sqrt((E_LPM*k)/(8.*E0*(E0-k)))

c ----- h(x) -----
      h = log(s_d)/log(sqrt(2.)*ls1)

c ----- xi(x) -----
      if (s_d.le.sq2s1) then
        xi = 2.
      else if (sq2s1.lt.s_d .and. s_d.le.1.) then
        xi = 1. + h - 0.08*(1.-h)*(1.-(1.-h)**2.)/log(sqrt(2.)*ls1)
      else
        xi = 1.
      end if

c ----- s(x) -----
      s = s_d/(xi)**(1./2.)

c ----- phi(x) -----
      if (s.le.0.01) then
        phi = 6.*s
      else if (0.01.lt.s .and. s.lt.2.) then
        phi = 1. - exp(-6.*s*(1.+(3.-PI)*s)+s**3.
     *                 /(0.623+0.796*s+0.658*s**2.))
      else
        phi = 1.
      end if

c ----- psi(x) -----
      if (s.le.0.01) then
        psi = 4.*s
      else if(0.01.lt.s .and. s.lt.2.) then
        psi = 1. - exp(-4.*s-8.*s**2.
     *                 /(1.+3.96*s+4.97*s**2.-0.05*s**3.+7.5*s**4.))
      else
        psi = 1.
      end if
c ----- G(x) -----
      G = 3.*psi - 2.*phi

c ---------------------- Dielectric factor ----------------------------
c     # Plank's constant/2*PI [MeV s]
      h_bar = 6.582e-22

c     # speed of light [cm/sec.]
      l_c = 2.997925e+10

c     # electron mass [g]
      l_m_e = 9.1091e-28

c     # electron mass c**2 [g]
      l_m = l_m_e * l_c**2. / (1.602e-19 * 1.0e+13)

c     # electronic charge [esu]
      l_e = 4.80298e-10

c     # atomic number per unit volume
      l_n = AN * RHO / l_WA

      omega = sqrt(4.*PI*l_n*Z*l_e**2./l_m_e)

      gmm = E0 / l_m
      k_p = gmm * h_bar * omega
      GM = 1. + k_p**2. / k**2.
      s_GM = s * GM

c     write(51,*)"Z, E0, k_p : ",Z,E0,k_p

c ----- phi_g(x) and Psi_g(x) -----
      if(s_GM.le.0.01) then
        phi_g = 6.*s_GM
        psi_g = 4.*s_GM
      else if(0.01.lt.s_GM .and. s_GM.lt.2.) then
        phi_g = 1. - exp(-6.*s_GM*(1.+(3.-PI)*s_GM)+s_GM**3.
     *                   /(0.623+0.796*s_GM+0.658*s_GM**2.))
        psi_g = 1. - exp(-4.*s_GM-8.*s_GM**2.
     *         /(1.+3.96*s_GM+4.97*s_GM**2.-0.05*s_GM**3.+7.5*s_GM**4.))
      else
        phi_g = 1.
        psi_g = 1.
      end if

c ---------- Migdal plus dielectric cross section ----------
      mgsigma = 1./3.*xi * ( y**2.*(3.*psi_g - 2.*phi_g)
     *          + 2.*(1.+(1.-y)**2.)*phi_g ) / GM

c -------------------- End of LPM cross section -----------------------

c ---------- LPM/BH-EGS -----------
      ratio_mgbh = mgsigma/bhsigma

                                                     ! ----------------
      return                                         ! Return to collis
                                                     ! ----------------
      end
