!----------------------------- ratio_mgop.f ----------------------------
!-----------------------------------------------------------------------
!  PROGRAMMERS: Y. Kirihara                                            *
!               Department of Accelerator Science,                     *
!               The Graduate University for Advanced Studies,          *
!               1-1, Oho, Tsukuba, Ibaraki, 305-0801                   *
!               Japan                                                  *
!                                                                      *
!               E-mail: kyoichi@post.kek.jp                            *
!-----------------------------------------------------------------------
      subroutine rmgop(k0,E,ratio_mgop)


      implicit none

      include 'include/egs5_h.f' ! COMMONs required by EGSD5 code
      include 'include/egs5_media.f'
      include 'include/egs5_brempr.f'
      include 'include/egs5_uphiot.f'
      include 'include/egs5_useful.f'
      real*8 AN,FSC,R0,RLC,k0,E,v,E_LPM,ls1,s_d,h,xi,s,phi,psi,G,sq2s1,
     *       mgsigma_p,opsigma_p,ratio_mgop
      real*8 Z_AB,Z_f,A_d,a,l_delta,delta,f_c,L_rad_d,L_rad,xi_bh,E_p,
     *       phi1_bh,phi2_bh,f1_bh,f2_bh,f3_bh
c ------------------------------------------------------------------
c Branching ratio as a function of an element.
c ------------------------------------------------------------------
      ! for Branching ratio
      integer i
      real*8 rnnow                                       ! Arguments
      real*8 sum_pz,sum_z,r_pz(MXEL),
     * ele_pdf(MXEL),ele_cdf(MXEL)
      real*8 Z,l_WA,RHO,RHOTBL(100)

      data RHOTBL/0.0808,0.19,0.534,1.85,2.5,2.26,1.14,1.568,1.5,1.0, 0.
     *9712,1.74,2.702,2.4,1.82,2.07,2.2,1.65,0.86,1.55,3.02,4.54, 5.87,7
     *.14,7.3,7.86,8.71,8.90,8.9333,7.140,5.91,5.36,5.73,4.80, 4.2,3.4,1
     *.53,2.6,4.47,6.4,8.57,9.01,11.50,12.20,12.50,12.,10.5, 8.65,7.30,7
     *.31,6.684,6.24,4.93,2.7,1.873,3.5,6.15,6.90,6.769, 7.007, 1. ,7.54
     *,5.17,7.87,8.25,8.56,8.80,9.06,9.32,6.96,9.85, 11.40,16.60,19.30,2
     *0.53,22.48,22.42,21.45,19.30,14.19,11.85, 11.34,9.78,9.30, 1. ,4.,
     * 1. ,5., 1. ,11.0,15.37,18.90, 20.5,19.737,11.7,7.,1. , 1. , 1. ,
     *1./

      sum_pz = 0.d0
      sum_z = 0.d0
      do i=1,nne(medium)
        sum_pz = sum_pz + pz(medium,i)
      end do

      do i=1,nne(medium)
        r_pz(i) = pz(medium,i)/sum_pz
        sum_z = sum_z + r_pz(i)*zelem(medium,i)*(zelem(medium,i)+1.)
      end do

      do i=1,nne(medium)
        ele_pdf(i) = r_pz(i)*zelem(medium,i)*(zelem(medium,i)+1.)/sum_z
      end do
c     -----------------------
c     Calculate cdf from pdf
c     -----------------------

      ele_cdf(1) = ele_pdf(1)
      do i=2,nne(medium)
        ele_cdf(i) = ele_cdf(i-1) + ele_pdf(i)
      end do

      call randomset(rnnow)
      do i=1,nne(medium)
        if (rnnow.le.ele_cdf(i)) then
          go to 100
        end if
      end do
100   continue

      Z = zelem(medium,i)
      l_WA = wa(medium,i)
      RHO = RHOTBL(int(Z))

c -------------------- Start of EGS-BH cross section ---------------
c According to the EGS5 CODE SYSTEM p.37 - 54
c ------------------------------------------------------------------
      AN = 6.02214199D+23          ! Avogadro's number
      FSC = 7.297360155253394E-003 ! fine structure constant
      R0 = 2.817943337446684E-013  ! Electron radius

      ! Eq. (2.53) from the EGS5 CODE SYSTEM
      a = FSC*Z
      f_c = a**2.*((1.+a**2.)**(-1.) + 0.20206 - 0.0369*a**2.
     *      + 0.0083*a**4. - 0.002*a**6.)

      ! Eq. (2.57) from the EGS5 CODE SYSTEM
      if (Z.eq.1.) then
        L_rad = 5.310
      else if (Z.eq.2.) then
        L_rad = 4.790
      else if (Z.eq.3.) then
        L_rad = 4.740
      else if (Z.eq.4.) then
        L_rad = 4.710
      else
        L_rad = log(184.15*Z**(-1./3.))
      end if

      ! Eq. (2.57) from the EGS5 CODE SYSTEM
      if (Z.eq.1.) then
        L_rad_d = 6.144
      else if (Z.eq.2.) then
        L_rad_d = 5.621
      else if (Z.eq.3.) then
        L_rad_d = 5.805
      else if (Z.eq.4.) then
        L_rad_d = 5.924
      else
        L_rad_d = log(1194.*Z**(-2./3.))
      end if

      ! Eq. (2.57) from the EGS5 CODE SYSTEM
      xi_bh = L_rad_d /(L_rad - f_c)

      ! No. 80 in Tab. 2.1 from the EGS5 CODE SYSTEM
      Z_AB = Z*(Z+xi_bh)*L_rad
      ! No. 40 in Tab. 2.1 from the EGS5 CODE SYSTEM
      Z_f = Z*(Z+xi_bh)*f_c

      ! when E0 > 50, A' = 1.
      A_d = 1.0

      ! RLC from PEGS data
c     RLC = 1./( (AN*RHO/l_WA)*4.0*FSC*R0**2*(Z_AB-Z_F) )
      RLC = rlcm(medium)

      E_p = k0 - E
      l_delta = k0*RM/(2.*E_p*E)
      delta = 136.*Z**(-1./3.)*2.*l_deltA

c ----- phi1 -----
      if(delta.le.1.) then
        phi1_bh = 20.867 - 3.242*delta + 0.625*delta**2.
      else
        phi1_bh = 21.12 - 4.184*log(delta+0.952)
      end if

c ----- phi2 -----
      if(delta.le.1.) then
        phi2_bh = 20.029 - 1.930*delta + 0.086*delta**2.
      else
        phi2_bh = 21.12 - 4.184*log(delta+0.952)
      end if

c ----- first factor -----
      f1_bh = a_d * Z*(Z+xi_bh) /(4.*(z_ab-z_f)) /k0**3.

c ----- second factor -----
      if (k0.ge.50.) then
        f2_bh = (E_p**2.+E**2.) * (phi1_bh - 4./3.*log(Z) - 4.*f_c)
      else
        f2_bh = (E_p**2.+E**2.) * (phi1_bh - 4./3.*log(Z))
      end if

c ----- third factor -----
      if (k0.ge.50.) then
        f3_bh = 2./3.*E_p*E * (phi2_bh - 4./3.*log(Z) - 4.*f_c)
      else
        f3_bh = 2./3.*E_p*E * (phi2_bh - 4./3.*log(Z))
      end if

c ---------- Original PEGS cross section for pair production ----------
      opsigma_p = f1_bh * ( f2_bh + f3_bh ) * k0

c -------------------- End of BH cross section ------------------------

c -------------------- Start of LPM cross section ---------------------
      E_LPM = 7.7e+6*RLC          ! E_LPM [MeV]

      ls1 = (Z**(1./3.)/191.)**2.
      v = E/k0                    ! E/k0 e-(e+) / photon
      sq2s1 = sqrt(2.)*ls1

c ----- s' (x) -----
      s_d = sqrt((E_LPM*k0)/(8.*E*(k0-E)))

c ----- h(x) -----
      h = log(s_d)/log(sqrt(2.)*ls1)

c ----- xi(x) -----
      if (s_d.le.sq2s1) then
        xi = 2.
      else if (sq2s1.lt.s_d .and. s_d.le.1.) then
        xi = 1. + h - 0.08*(1.-h)*(1.-(1.-h)**2.)/log(sqrt(2.)*ls1)
      else
        xi = 1.
      end if

c ----- s(x) -----
      s = s_d/(xi)**(1./2.)

c ----- phi(x) -----
      if (s.le.0.01) then
        phi = 6.*s
      else if (0.01.lt.s .and. s.lt.2.) then
        phi = 1. - exp(-6.*s*(1.+(3.-PI)*s)+s**3.
     *                 /(0.623+0.796*s+0.658*s**2.))
      else
        phi = 1.
      end if

c ----- psi(x) -----
      if (s.le.0.01) then
        psi = 4.*s
      else if(0.01.lt.s .and. s.lt.2.) then
        psi = 1. - exp(-4.*s-8.*s**2.
     *                 /(1.+3.96*s+4.97*s**2.-0.05*s**3.+7.5*s**4.))
      else
        psi = 1.
      end if

c ----- G(x) -----
      G = 3.*psi - 2.*phi

c ---------- Migdal cross section for pair production ----------
      mgsigma_p = 1./3.*xi * ( (3.*psi - 2.*phi)
     *          + 2.*(v**2. + (1.-v)**2.) * phi )
c -------------------- End of LPM cross section --------------------

c ----- Migdal/Bethe-Heitler
      ratio_mgop = mgsigma_p/opsigma_p
c -------------------- End of BH cross section ------------------------

                                                      ! ----------------
      return                                          ! Return to collis
                                                      ! ----------------
      end
